@@ -880,7 +880,9 @@ gff_load_metrics (GnomeFontFace *face, gint glyph)
 
 	gi = face->glyphs + glyph;
 
-	FT_Load_Glyph (face->ft_face, glyph, FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP);
+	FT_Load_Glyph (face->ft_face, glyph,
+		       FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING |
+		       FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM);
 
         gi->bbox.x0 = -face->ft_face->glyph->metrics.horiBearingX * face->ft2ps;
 	gi->bbox.y1 = face->ft_face->glyph->metrics.horiBearingY * face->ft2ps;
