@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  gp-fontmap.c: fontmap implementation
+ *  gnome-fontmap.c: fontmap implementation
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -18,6 +18,8 @@
  *
  *  Authors:
  *    Lauris Kaplinski <lauris@ximian.com>
+ *    Chema Celorio <chema@celorio.com>
+ *    Tambet Ingo <tambet@ximian.com>
  *
  *  Copyright (C) 2000-2001 Ximian, Inc.
  *
@@ -32,15 +34,6 @@
 #include <fontconfig/fontconfig.h>
 #include "gnome-fontmap.h"
 
-/*
- * We parse following locations for fontmaps:
- *
- * $FONTMAPDIR_STATIC ($prefix/share/gnome/fonts)
- * $FONTMAPDIR_DYNAMIC ($sysconfdir/gnome/fonts)
- * $HOME/.gnome/fonts
- *
- */
-
 static GPFontMap *gp_fontmap_load (void);
 
 static void gp_fontmap_ref (GPFontMap * map);
@@ -172,40 +165,87 @@ convert_fc_weight (gint i)
 static GPFontEntry *
 fcpattern_to_gp_font_entry (FcPattern *font)
 {
-	GPFontEntry *e;
+	GPFontEntryType font_type = GP_FONT_ENTRY_UNKNOWN;
 	GPFontEntryTT *tt;
+	GPFontEntryT1 *t1;
+	GPFontEntry *e;
 	FcResult result;
-	FcChar8 *str;
-	int i;
+	FcChar8 *fc_family, *fc_style, *fc_file;
+	int italic_angle, weight;
 	gchar *c;
 
-	tt = g_new0 (GPFontEntryTT, 1);
-	e = (GPFontEntry *)tt;
+	/* Get Family, Style & Filename */
+	result = FcPatternGetString (font, FC_FAMILY, 0, &fc_family);
+	if (result != FcResultMatch || fc_family == NULL) {
+		g_warning ("Can't create GPFontEntry because we could not get a FC_FAMILY for the font\n");
+		return NULL;
+	}
+	result = FcPatternGetString (font, FC_STYLE, 0, &fc_style);
+	if (result != FcResultMatch || fc_style == NULL) {
+		g_warning ("Can't create GPFontEntry for %s because we could not get its FC_STYLE\n",
+			   fc_family);
+		return NULL;
+	}
+	result = FcPatternGetString (font, FC_FILE, 0, &fc_file);
+	if (result != FcResultMatch || fc_file == NULL) {
+		g_warning ("Can't create GPFontEntry for %s-%s because we could not get its FC_FILE\n",
+			   fc_family, fc_style);
+		return NULL;
+	}
+	italic_angle = 0;
+	FcPatternGetInteger (font, FC_SLANT, 0, &italic_angle);
 
-	e->type = GP_FONT_ENTRY_TRUETYPE;
-	e->refcount = 1;
-	e->face = NULL;
-
-	result = FcPatternGetString (font, FC_FAMILY, 0, &str);
-	if (result == FcResultMatch)
-		e->familyname = g_strdup (str);
-
-	result = FcPatternGetString (font, FC_STYLE, 0, &str);
-	if (result == FcResultMatch) {
-		e->speciesname = g_strdup (str);
-		e->weight = g_strdup (str);
+	/* Determine what type of font we've got */
+	c = fc_file + strlen (fc_file) - 4;
+	if (strcasecmp (c, ".pfb") == 0)
+		font_type = GP_FONT_ENTRY_TYPE1;
+	else if (strcasecmp (c, ".pfa") == 0)
+		font_type = GP_FONT_ENTRY_TYPE1;
+	else if (strcasecmp (c, ".ttf") == 0)
+		font_type = GP_FONT_ENTRY_TRUETYPE;
+	else {
+		return NULL;
+		g_warning ("Can't create GPFontEntry for %s-%s because the extension could not be recognized (%s)\n",
+			   fc_family, fc_style, c);
+		return NULL;
 	}
 
-	result = FcPatternGetInteger (font, FC_WEIGHT, 0, &i);
+	switch (font_type) {
+	case GP_FONT_ENTRY_TRUETYPE:
+		tt = g_new0 (GPFontEntryTT, 1);
+		e = (GPFontEntry *) tt;
+		e->type = GP_FONT_ENTRY_TRUETYPE;
+		tt->ttf.name = g_strdup (fc_file);
+		tt->facenum = 0; /* FIMXE: Is this correct? (Chema) */
+		break;
+	case GP_FONT_ENTRY_TYPE1:
+		t1 = g_new0 (GPFontEntryT1, 1);
+		e = (GPFontEntry *) t1;
+		e->type = GP_FONT_ENTRY_TYPE1;
+		t1->pfb.name = g_strdup (fc_file);
+		t1->afm.name = NULL;
+		break;
+	default:
+		g_warning ("Can't create GPFontEntry, invalid font_type)\n");
+		return NULL;
+	}
+
+	e->refcount = 1;
+	e->face = NULL;
+	e->speciesname = g_strdup (fc_style);
+	e->weight = g_strdup (fc_style);
+	e->familyname = g_strdup (fc_family);
+	e->version = g_strdup ("1.0"); /* FIMXE: Is this correct? (Chema) */
+	e->name    = g_strdup_printf ("%s %s", e->familyname, e->weight);
+	e->italic_angle = italic_angle;
+	
+	result = FcPatternGetInteger (font, FC_WEIGHT, 0, &weight);
 	if (result == FcResultMatch)
-		e->Weight = convert_fc_weight (i);
+		e->Weight = convert_fc_weight (weight);
 	else
 		e->Weight = GNOME_FONT_REGULAR;
 
-	e->version = g_strdup ("1.0");
-
-	e->name   = g_strdup_printf ("%s %s", e->familyname, e->weight);
-	e->psname = g_strdup_printf ("%s-%s", e->familyname, e->weight);
+	e->psname  = g_strdup_printf ("%s-%s", e->familyname, e->weight);
 	c = e->psname;
 	while (*c != '\0') {
 		if (*c == ' ')
@@ -213,14 +253,6 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 		c++;
 	}
 
-	result = FcPatternGetInteger (font, FC_SLANT, 0, &i);
-	if (result == FcResultMatch)
-		e->ItalicAngle = i;
-
-	result = FcPatternGetString (font, FC_FILE, 0, &str);
-	if (result == FcResultMatch)
-		tt->ttf.name = g_strdup (str);
-
 	return e;
 }
 
@@ -231,6 +263,8 @@ fcpattern_to_gp_font_entry_alias (FcPattern *font, const gchar *family, const gc
 	gchar *c;
 
 	e = fcpattern_to_gp_font_entry (font);
+	if (e == NULL)
+		return NULL;
 
 	/* Override font name and style */
 	if (e->name)
@@ -262,6 +296,10 @@ gp_fontmap_add_aliases (GPFontMap *map)
 	FcResult res;
 	gint i, j;
 
+	/* If no fonts could be loaded, we can't create aliases */
+	if (map == NULL)
+		return;
+
 	for (i = 0; aliases[i]; i++) {
 		for (j = 0; styles[j]; j++) {
 			match_pattern = FcPatternBuild (NULL,
@@ -291,6 +329,40 @@ gp_fontmap_add_aliases (GPFontMap *map)
 	}
 }
 
+/**
+ * gp_fontmap_check_duplicates:
+ * @map: 
+ * 
+ * This function makes sure that the e->psname is unique for each font
+ * it appends a dash + 3 digits if it find a duplicate. It grows exponentially which
+ * might be a problem for system with a lot of fonts.
+ **/
+static void
+gp_fontmap_check_duplicates (GPFontMap *map)
+{
+	GSList *list_1, *list_2;
+	static gint i;
+
+	list_1 = map->fonts;
+	while (list_1) {
+		GPFontEntry *e1 = list_1->data;
+		list_2 = map->fonts;
+		while (list_2) {
+			GPFontEntry *e2 = list_2->data;
+			if (list_2 != list_1) {
+				if (strcmp (e1->psname, e2->psname) == 0) {
+					gchar *new;
+					new = g_strdup_printf ("%s-%.3d", e2->psname, i++);
+					g_free (e2->psname);
+					e2->psname = new;
+				}
+			}
+			list_2 = list_2->next;
+		}
+		list_1 = list_1->next;
+	}
+}
+
 static void
 gp_fontmap_load_fontconfig (GPFontMap *map)
 {
@@ -316,6 +388,7 @@ gp_fontmap_load_fontconfig (GPFontMap *map)
 	}
 
 	gp_fontmap_add_aliases (map);
+	gp_fontmap_check_duplicates (map);
 }
 
 static GPFontMap *
@@ -480,21 +553,26 @@ gp_font_entry_unref (GPFontEntry * entry)
 		switch (entry->type) {
 		case GP_FONT_ENTRY_TYPE1_ALIAS:
 			t1a = (GPFontEntryT1Alias *) entry;
-			if (t1a->alias) g_free (t1a->alias);
+			if (t1a->alias)
+				g_free (t1a->alias);
 		case GP_FONT_ENTRY_TYPE1:
 			t1 = (GPFontEntryT1 *) entry;
-			if (t1->afm.name) g_free (t1->afm.name);
-			if (t1->pfb.name) g_free (t1->pfb.name);
+			if (t1->afm.name)
+				g_free (t1->afm.name);
+			if (t1->pfb.name)
+				g_free (t1->pfb.name);
 			break;
 		case GP_FONT_ENTRY_ALIAS:
 			break;
 		case GP_FONT_ENTRY_TRUETYPE:
 			tt = (GPFontEntryTT *) entry;
-			if (tt->ttf.name) g_free (tt->ttf.name);
+			if (tt->ttf.name)
+				g_free (tt->ttf.name);
 			break;
 		case GP_FONT_ENTRY_SPECIAL:
 			s = (GPFontEntrySpecial *) entry;
-			if (s->file.name) g_free (s->file.name);
+			if (s->file.name)
+				g_free (s->file.name);
 			while (s->additional) {
 				g_free (s->additional->data);
 				s->additional = g_slist_remove (s->additional, s->additional->data);
