@@ -18,24 +18,103 @@
  *
  *  Authors:
  *    Lauris Kaplinski <lauris@helixcode.com>
+ *    Chema Celorio <chema@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc.
+ *  Copyright 2001-2003 Ximian, Inc.
  *
  */
 
-#define __GNOME_PRINT_CONFIG_C__
+#include <config.h>
 
 #include <ctype.h>
 #include <stdlib.h>
 #include <locale.h>
-#include "gpa/gpa-node-private.h"
+
+#include <libgnomeprint/gpa/gpa-config.h>
+
 #include "gnome-print-config.h"
+#include "gnome-print-config-private.h"
+
+/*
+ * I changed GnomePrintConfig form being a boxed type to a GObject,
+ * because app developers are going to g_object_unref anyways  Move the
+ * following macros to the .h file and make gnome_print_config_[un]ref a macro rather
+ * than a function call for GNOME 2.4 (Chema)
+ */
+#define GNOME_TYPE_PRINT_CONFIG         (gnome_print_config_get_type ())
+#define GNOME_PRINT_CONFIG(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GNOME_TYPE_PRINT_CONFIG, GnomePrintConfig))
+#define GNOME_PRINT_CONFIG_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), GNOME_TYPE_PRINT_CONFIG, GnomePrintConfigClass))
+#define GNOME_IS_PRINT_CONFIG(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNOME_TYPE_PRINT_CONFIG))
+#define GNOME_IS_PRINT_CONFIG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GNOME_TYPE_PRINT_CONFIG))
+#define GNOME_PRINT_CONFIG_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GNOME_TYPE_PRINT_CONFIG, GnomePrintConfigClass))
+
+typedef struct _GnomePrintConfigClass GnomePrintConfigClass;
 
 struct _GnomePrintConfig {
-	gint refcount;
+	GObject parent;
 	GPANode *node;
 };
 
+struct _GnomePrintConfigClass {
+	GObjectClass parent_class;
+};
+
+static void gnome_print_config_class_init (GnomePrintConfigClass *klass);
+static void gnome_print_config_init (GnomePrintConfig *pc);
+static void gnome_print_config_finalize (GObject *object);
+
+static GObjectClass *parent_class = NULL;
+
+GType
+gnome_print_config_get_type (void)
+{
+	static GType type;
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (GnomePrintConfigClass),
+			NULL, NULL,
+			(GClassInitFunc) gnome_print_config_class_init,
+			NULL, NULL,
+			sizeof (GnomePrintConfig),
+			0,
+			(GInstanceInitFunc) gnome_print_config_init
+		};
+		type = g_type_register_static (G_TYPE_OBJECT, "GnomePrintConfig", &info, 0);
+	}
+	return type;
+}
+
+static void
+gnome_print_config_class_init (GnomePrintConfigClass *klass)
+{
+	GObjectClass *object_class;
+
+	object_class = (GObjectClass*) klass;
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = gnome_print_config_finalize;
+}
+
+static void
+gnome_print_config_init (GnomePrintConfig *config)
+{
+	config->node = NULL;	
+}
+
+static void
+gnome_print_config_finalize (GObject *object)
+{
+	GnomePrintConfig *config;
+
+	config = GNOME_PRINT_CONFIG (object);
+
+	config->node = gpa_node_unref (config->node);
+	
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 GPANode *
 gnome_print_config_get_node (GnomePrintConfig *config)
 {
@@ -49,10 +128,8 @@ gnome_print_config_default (void)
 {
 	GnomePrintConfig *config;
 
-	config = g_new (GnomePrintConfig, 1);
-
-	config->refcount = 1;
-	config->node = gpa_defaults ();
+	config = g_object_new (GNOME_TYPE_PRINT_CONFIG, NULL);
+	config->node = (GPANode *) gpa_config_new ();
 
 	return config;
 }
@@ -61,8 +138,9 @@ GnomePrintConfig *
 gnome_print_config_ref (GnomePrintConfig *config)
 {
 	g_return_val_if_fail (config != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), NULL);
 
-	config->refcount += 1;
+	g_object_ref (G_OBJECT (config));
 
 	return config;
 }
@@ -71,14 +149,10 @@ GnomePrintConfig *
 gnome_print_config_unref (GnomePrintConfig *config)
 {
 	g_return_val_if_fail (config != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), NULL);
 
-	config->refcount -= 1;
-
-	if (config->refcount < 1) {
-		config->node = gpa_node_unref (config->node);
-		g_free (config);
-	}
-
+	g_object_unref (G_OBJECT (config));
+	
 	return NULL;
 }
 
@@ -89,9 +163,8 @@ gnome_print_config_dup (GnomePrintConfig *old_config)
 
 	g_return_val_if_fail (old_config != NULL, NULL);
 
-	config = g_new (GnomePrintConfig, 1);
-
-	config->refcount = 1;
+	config = g_object_new (GNOME_TYPE_PRINT_CONFIG, NULL);
+	
 	config->node = gpa_node_duplicate (old_config->node);
 
 	return config;
@@ -169,11 +242,12 @@ gnome_print_config_get_int (GnomePrintConfig *config, const guchar *key, gint *v
 
 	if (v != NULL) {
 		gchar *loc;
-		loc = setlocale (LC_NUMERIC, NULL);
+		loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 		setlocale (LC_NUMERIC, "C");
 		*val = atoi (v);
 		g_free (v);
 		setlocale (LC_NUMERIC, loc);
+		g_free (loc);
 		return TRUE;
 	}
 	
@@ -194,11 +268,12 @@ gnome_print_config_get_double (GnomePrintConfig *config, const guchar *key, gdou
 
 	if (v != NULL) {
 		gchar *loc;
-		loc = setlocale (LC_NUMERIC, NULL);
+		loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 		setlocale (LC_NUMERIC, "C");
 		*val = atof (v);
 		g_free (v);
 		setlocale (LC_NUMERIC, loc);
+		g_free (loc);
 		return TRUE;
 	}
 	
@@ -221,10 +296,11 @@ gnome_print_config_get_length (GnomePrintConfig *config, const guchar *key, gdou
 
 	if (v != NULL) {
 		guchar *loc, *e;
-		loc = setlocale (LC_NUMERIC, NULL);
+		loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 		setlocale (LC_NUMERIC, "C");
 		*val = strtod (v, (gchar **) &e);
 		setlocale (LC_NUMERIC, loc);
+		g_free (loc);
 		if (e == v) {
 			g_free (v);
 			return FALSE;
@@ -234,7 +310,8 @@ gnome_print_config_get_length (GnomePrintConfig *config, const guchar *key, gdou
 			c_unit = GNOME_PRINT_PS_UNIT;
 		} else {
 			c_unit = gnome_print_unit_get_by_abbreviation (e);
-			if (!c_unit) c_unit = gnome_print_unit_get_by_name (e);
+			if (!c_unit)
+				c_unit = gnome_print_unit_get_by_name (e);
 		}
 		g_free (v);
 		if (c_unit == NULL)
@@ -270,10 +347,11 @@ gnome_print_config_set_int (GnomePrintConfig *config, const guchar *key, gint va
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (*key != '\0', FALSE);
 
-	loc = setlocale (LC_NUMERIC, NULL);
+	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	g_snprintf (c, 128, "%d", val);
 	setlocale (LC_NUMERIC, loc);
+	g_free (loc);
 
 	return gnome_print_config_set (config, key, c);
 }
@@ -288,10 +366,11 @@ gnome_print_config_set_double (GnomePrintConfig *config, const guchar *key, gdou
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (*key != '\0', FALSE);
 
-	loc = setlocale (LC_NUMERIC, NULL);
+	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	g_snprintf (c, 128, "%g", val);
 	setlocale (LC_NUMERIC, loc);
+	g_free (loc);
 
 	return gnome_print_config_set (config, key, c);
 }
@@ -307,12 +386,44 @@ gnome_print_config_set_length (GnomePrintConfig *config, const guchar *key, gdou
 	g_return_val_if_fail (*key != '\0', FALSE);
 	g_return_val_if_fail (unit != NULL, FALSE);
 
-	loc = setlocale (LC_NUMERIC, NULL);
+	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	g_snprintf (c, 128, "%g%s", val, unit->abbr);
 	setlocale (LC_NUMERIC, loc);
+	g_free (loc);
 
 	return gnome_print_config_set (config, key, c);
 }
 
 
+gchar *
+gnome_print_config_to_string (GnomePrintConfig *config, guint flags)
+{
+	g_return_val_if_fail (config != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), NULL);
+
+	return gpa_config_to_string (GPA_CONFIG (config->node), flags);
+}
+
+GnomePrintConfig *
+gnome_print_config_from_string (const gchar *str,
+				guint flags)
+{
+	GnomePrintConfig *config;
+
+	g_return_val_if_fail (str != NULL, NULL);
+
+	config = g_object_new (GNOME_TYPE_PRINT_CONFIG, NULL);
+	config->node = (GPANode *) gpa_config_from_string (str, flags);
+
+	return config;
+}
+
+void
+gnome_print_config_dump (GnomePrintConfig *config)
+{
+	g_return_if_fail (config != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_CONFIG (config));
+
+	gpa_utils_dump_tree (config->node, 1);
+}
