@@ -1,23 +1,60 @@
-#ifndef __GNOME_PRINT_H__
-#define __GNOME_PRINT_H__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Abstract base class for drivers
+ *  gnome-print.c: Abstract base class of gnome-print drivers
  *
- * Authors:
- *   Raph Levien (raph@acm.org)
- *   Miguel de Icaza (miguel@kernel.org)
- *   Lauris Kaplinski <lauris@ximian.com>
- *   Chema Celorio (chema@celorio.com)
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * Copyright (C) 1999-2001 Ximian, Inc. and authors
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Raph Levien <raph@acm.org>
+ *    Miguel de Icaza <miguel@kernel.org>
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *    Chema Celorio <chema@celorio.com>
+ *
+ *  Copyright (C) 1999-2001 Ximian Inc. and authors
  *
  */
 
+#ifndef __GNOME_PRINT_H__
+#define __GNOME_PRINT_H__
+
 #include <glib.h>
 
 G_BEGIN_DECLS
 
+/*
+ * Notes
+ *
+ * GnomePrintContexts are both actual drivers and various frontends to
+ * further job-processing objects. The API is always the same. Output
+ * area is not constrained, so applications have to decide effective
+ * page size themselves. The job may or may not be clipped to effective
+ * page size in printing pipeline, so in multipage layouts your output
+ * may be run out of component page area. If unsure, clip full page
+ * manually, as first graphic operation.
+ *
+ * All graphic operators have to be invoked between beginpage/showpage
+ * pairs, doing otherwise is error and given method is ignored.
+ *
+ * If printing is completed, print context should be closed (to flush
+ * all data). Closing does not unref print context, so if you have
+ * obtained reference to it, release that after closing. Destroying
+ * unclosed context discards all printed data (think about it as
+ * canceling print job).
+ *
+ */
+
 #include <glib-object.h>
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/gnome-font.h>
@@ -26,11 +63,14 @@ G_BEGIN_DECLS
 #include <libart_lgpl/art_bpath.h>
 
 /*
+ * Result (error) codes
+ *
  * These are currently defined values for gnome_print_xxx results
  * feel free to add more, if those make sense.
  * The rule is, that errors have to be negative, as several
- * methods return num bytes written, >=0 has to be interpreted
+ * methods return num bytes written, >= 0 has to be interpreted
  * as success.
+ *
  */
 
 typedef enum {
@@ -51,16 +91,18 @@ typedef enum {
 
 typedef struct _GnomePrintContext GnomePrintContext;
 
-/*
- * INTERNALS ARE PRIVATE UNTIL STABILIZED
- */
-
 GType gnome_print_context_get_type (void);
 
 /*
  * Create new printing context from config. You have to have set
  * all settings beforehand, as changing config of existing context
  * has undefined result.
+ *
+ * Also, if creating context by hand, it completely ignores layout and
+ * orientation value. If you need those, use GnomePrintMaster. The
+ * latter also can create output context for you, so in most cases
+ * you may want to ignore gnome_print_context_new at all.
+ *
  */
 
 GnomePrintContext *gnome_print_context_new (GnomePrintConfig *config);
@@ -72,6 +114,15 @@ GnomePrintContext *gnome_print_context_new (GnomePrintConfig *config);
 
 gint gnome_print_context_close (GnomePrintContext *pc);
 
+/*
+ * Printing methods
+ *
+ * These behave similarly to standard PostScript operators, with some
+ * slight differences. Most importantly ALL path output operations
+ * (fill, stroke, clip) clear currentpath.
+ *
+ */
+
 /* Path manipulation */
 
 gint gnome_print_newpath (GnomePrintContext *pc);
@@ -102,7 +153,7 @@ gint gnome_print_eoclip (GnomePrintContext *pc);
 
 gint gnome_print_concat (GnomePrintContext *pc, const gdouble *matrix);
 gint gnome_print_scale (GnomePrintContext *pc, gdouble sx, gdouble sy);
-gint gnome_print_rotate (GnomePrintContext *pc, gdouble theta_in_degrees);
+gint gnome_print_rotate (GnomePrintContext *pc, gdouble theta);
 gint gnome_print_translate (GnomePrintContext *pc, gdouble x, gdouble y);
 
 /* Stack */
@@ -148,6 +199,7 @@ gint gnome_print_showpage (GnomePrintContext *pc);
  * Callback is wrapped between beginpage/endpage pair internally,
  * so you cannot mix callback with ordinary PDL.
  */
+#ifdef GNOME_PRINT_ENABLE_PAGE_CALLBACKS
 
 typedef gint (* GnomePrintPageCallback) (GnomePrintContext *ctx, gpointer pagedata, gpointer docdata);
 
@@ -156,10 +208,12 @@ gint gnome_print_page_callback         (GnomePrintContext *ctx, GnomePrintPageCa
 
 gint gnome_print_page_callback_closure (GnomePrintContext *ctx, const guchar *name,
 					gpointer pagedata, gpointer docdata, GClosure *closure);
+#endif
 
 /*
- * Convenience methods
+ * Convenience methods for printing straight lines and rectangles.
  */
+
 gint gnome_print_line_stroked (GnomePrintContext *pc, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
 gint gnome_print_rect_stroked (GnomePrintContext *pc, gdouble x, gdouble y, gdouble width, gdouble height);
 gint gnome_print_rect_filled  (GnomePrintContext *pc, gdouble x, gdouble y, gdouble width, gdouble height);
