@@ -1,8 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
-#ifndef __GPA_NODE_PRIVATE_H__
-#define __GPA_NODE_PRIVATE_H__
-
 /*
  *  gpa-node.h:
  *
@@ -24,10 +20,14 @@
  *    Lauris Kaplinski <lauris@ximian.com>
  *    Jose M. Celorio <chema@ximian.com>
  *
- *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
+ *  Copyright (C) 2000-2003 Ximian, Inc.
  *
  */
 
+
+#ifndef __GPA_NODE_PRIVATE_H__
+#define __GPA_NODE_PRIVATE_H__
+
 #include <glib.h>
 
 G_BEGIN_DECLS
@@ -35,23 +35,27 @@ G_BEGIN_DECLS
 #include <glib-object.h>
 #include "gpa-utils.h"
 
-#define GPA_NODE_MODIFIED_FLAG ( 1 << 5 )
-
 #define GPA_NODE_FLAGS(n)         (GPA_NODE(n)->flags)
 #define GPA_NODE_SET_FLAGS(n,f)   (GPA_NODE(n)->flags |= f)
 #define GPA_NODE_UNSET_FLAGS(n,f) (GPA_NODE(n)->flags &= (~f))
 
-#define GPA_NODE_ID(n)           (gpa_quark_to_string (GPA_NODE (n)->qid))
-#define GPA_NODE_ID_COMPARE(n,s) ((s) && (gpa_quark_try_string (s) == GPA_NODE (n)->qid))
-#define GPA_NODE_ID_EXISTS(n)    (GPA_NODE (n)->qid != 0)
+#define GPA_NODE_ID(n)           (n ? g_quark_to_string (GPA_NODE (n)->qid) : NULL)
+#define GPA_NODE_ID_COMPARE(n,s) ((s) && (g_quark_try_string (s) == GPA_NODE (n)->qid))
 #define GPA_NODE_PARENT(n)       (GPA_NODE (n)->parent)
 
+typedef enum {
+	FLAGS_NOT_USED_YET,
+} GPANodeFlags;
+
 struct _GPANode {
 	GObject object;
-	guint32 flags;
-	guint32 qid;
+	guint32 flags;  /* Future use */
+	GQuark qid;     /* quark id of the node */
+
+	/* Tree structure links*/
 	GPANode *parent;
 	GPANode *next;
+	GPANode *children;
 };
 
 struct _GPANodeClass {
@@ -60,9 +64,6 @@ struct _GPANodeClass {
 	guchar * (* get_value)  (GPANode *node);
 	gboolean (* set_value)  (GPANode *node, const guchar *value);
 
-	GPANode * (* get_child) (GPANode *node, GPANode *previous_child);
-	GPANode * (* lookup)    (GPANode *node, const guchar *path);
-
 	GPANode * (* duplicate) (GPANode *node);
 	gboolean  (* verify)    (GPANode *node);
 	
@@ -70,23 +71,15 @@ struct _GPANodeClass {
 	void     (* modified)   (GPANode *node, guint flags);
 };
 
-GType gpa_node_get_type (void);
-
+GType     gpa_node_get_type (void);
 GPANode * gpa_node_new (GType type, const guchar *id);
-GPANode * gpa_node_duplicate (GPANode *node);
+
 gboolean  gpa_node_verify    (GPANode *node);
+GPANode * gpa_node_duplicate (GPANode *node);
 GPANode * gpa_node_lookup    (GPANode *node, const guchar *path);
 
-/* Request that "modified" signal will be run from idle loop */
-void gpa_node_request_modified (GPANode *node, guint flags);
-void gpa_node_emit_modified    (GPANode *node, guint flags);
-
-/* fixme: */
-#if 0
-GSList *gpa_list_nodes (void);
-#endif
+void      gpa_node_emit_modified    (GPANode *node);
 
 G_END_DECLS
 
 #endif /* __GPA_NODE_PRIVATE_H__ */
-
